#ifndef INCLUDED_CURVE_EVALUATORS_H
#define INCLUDED_CURVE_EVALUATORS_H

#pragma warning(disable : 4786)  

#include <string>

#include <map>

// g++ doesn't support auto_ptr
#ifdef WIN32
#include "Singleton.h"
#endif

using namespace std;

class CurveEvaluator;

#ifndef WIN32
class CurveEvaluators;

class CurveEvaluators  
#else
class CurveEvaluators : public Singleton<CurveEvaluators>
#endif
{
	public:

		~CurveEvaluators(void);

		void addCurveEvaluator(const string & curve_name, CurveEvaluator * curve_evaluator);

		CurveEvaluator * getCurveEvaluator(const string & curve_name) const;

		typedef map<string, CurveEvaluator *> CurveNameEvaluatorMapping;

		typedef CurveNameEvaluatorMapping::const_iterator const_iterator;

		CurveNameEvaluatorMapping::const_iterator begin(void) const;
		CurveNameEvaluatorMapping::const_iterator end(void) const;

#ifndef WIN32
		static CurveEvaluators * getInstance(void) 
		{
			if (m_instance == NULL)
			{
				m_instance = new CurveEvaluators();
			}

			return m_instance;
		}
#endif

	private:

		void cleanupCurveEvaluators(void);

		CurveNameEvaluatorMapping m_curve_name_evaluator_mapping;

#ifndef WIN32
		static CurveEvaluators *m_instance;
#endif
};

#endif
