#ifndef INCLUDED_CURVES_H
#define INCLUDED_CURVES_H

#pragma warning(disable : 4786)

#include "Curve.h"

#include <iostream>

#include <vector>

// g++ doesn't support auto_ptr
#ifdef WIN32
#include "Singleton.h"
#endif

using namespace std;

#ifndef WIN32
class Curves;

class Curves  
#else
class Curves : public Singleton<Curves>
#endif
{
	public:

		Curves(void);

		Curves(istream & input_stream);

		void toStream(ostream & output_stream);

		void fromStream(istream & input_stream);

		void addCurve(const string & curve_name, const double & minimum_value, const double & maximum_value, const double & start_value);

		Curve * getCurve(const string & curve_name);

		typedef vector<Curve> CurveList;

		typedef CurveList::const_iterator const_iterator;

		const_iterator begin() const;
		const_iterator end() const;

		typedef CurveList::iterator iterator;

		iterator begin();
		iterator end();

		unsigned int getNumberOfCurves(void) const;

		void clearCurves(void);

		void setWrapControlPointsFlags(const bool & wrapping);

		void getClosestControlPointOnShownCurve(const Point & point, double * closest_control_point_distance, Curve * * closest_control_point_curve, unsigned int * closest_control_point_index);

		void getClosestShownCurve(const Point & point, double * closest_curve_distance, Curve * * closest_curve);

		void startZoomSelection(const double & left, const double & bottom);

		void finishZoomSelection(const double & right, const double & top);

		void pushZoom(void);

		void popZoom(void);

		bool canZoomOut(void) const;

		void changeAnimationLength(const double & animation_length);

		double getTime(const double & normalized_x) const;

		void drawAllCurves(void) const;

#ifndef WIN32
		static Curves * getInstance(void) 
		{
			if (m_instance == NULL)
			{
				m_instance = new Curves();
			}

			return m_instance;
		}
#endif

	private:

		CurveList m_curves;

#ifndef WIN32
		static Curves *m_instance;
#endif
};

ostream & operator<<(ostream & output_stream, Curves & curves);

istream & operator>>(istream & input_stream, Curves & curves);

#endif
