// 
// object.cpp
// 

#include <stdio.h>

#include "aniUI.h"
#include "object.h"
#include "glView.h"
#include "ctrlPanel.h"

extern CAniUI*  aniUI;

//UI controls
extern void group(char *);
extern void curve(int,char*,float,float,float, int groupID=0);
extern void scale(int,char*,float,float,float,float,int groupID=0);
extern void checkbox(int,char*,int,int groupID=0);
extern void menu(int,char*,int,char*,...);
extern void ptr(int,void*);
extern float get_control_f(int,int);
extern int   get_control_i(int,int);
extern bool  get_control_b(int,int);
extern void*  get_ptr(int,int);

extern float getCurrentTime();

extern int getUniqueID();
extern void addObject(int,Object*);


Object::Object(char* dllname, void *args)
{
	DLHANDLE dlh=DL_OpenLibrary(dllname);

	if (dlh==NULL) {
		m_bLoaded=false;
		return;
	}
	
	// bind functions
	if ((init=(lpInitFunc)DL_BindFunction(dlh, "init"))==NULL) {
		m_bLoaded=false;
		DL_CloseLibrary(dlh);
		return;
	}

	if ((redraw=(lpRedrawFunc)DL_BindFunction(dlh, "redraw_driver"))==NULL){
		m_bLoaded=false;
		DL_CloseLibrary(dlh);
		return;
	}

	if ((getName=(lpGetNameFunc)DL_BindFunction(dlh, "getExportModelName"))==NULL){
		m_bLoaded=false;
		DL_CloseLibrary(dlh);
		return;
	}

	lpAssignIDFunc assignID;
	if ((assignID=(lpAssignIDFunc)DL_BindFunction(dlh, "assignID"))==NULL){
		m_bLoaded=false;
		DL_CloseLibrary(dlh);
		return;
	}

	if ((patchControl=(lpPatchControlFunc)DL_BindFunction(dlh, "patchControl"))==NULL){
		m_bLoaded=false;
		DL_CloseLibrary(dlh);
		return;
	}

	m_bVisible=true;

	// get id
	id=getUniqueID();
	addObject(id, this);

	// get object name
	assignID(id);
	getName(objname);

	// call init for the object
	char *buf=new char[64];
	sprintf(buf, "%s<%d>", objname, id);

	m_pPanel=new ctrlPanel(100, 40, buf);
	m_pPanel->setObject(this);

	m_pPanel->makeCurrent();
	patchControl(group, curve, scale, checkbox, menu,
				get_control_f, get_control_i, get_control_b, 
				getCurrentTime, ptr, get_ptr);
	if (!init(args)) {
		m_bLoaded = false;	// can't be loaded
		return;
	}
	m_pPanel->completeInitialization();

	m_pPanel->setUI(aniUI);
	
	// first, show it such that the control values are set
	// then hide it since we havn't called it yet.
	m_pPanel->show();
	m_pPanel->hide();

	m_DLHandle = dlh;
	m_bLoaded = true;
}


Object::~Object()
{
	if (m_bLoaded)
		DL_CloseLibrary(m_DLHandle);
}

bool Object::isLoaded()
{
	return m_bLoaded;
}

void Object::setInit(lpInitFunc init_proc)
{
	init = init_proc;
}

void Object::setRedraw(lpRedrawFunc redraw_proc)
{
	redraw = redraw_proc;
}

float Object::getControlValue(int id) {
	return m_pPanel->getControlValue(id);
}

void* Object::getPtr(int id) {
	return m_pPanel->getPtr(id);
}

char *Object::getCurveName(int id) 
{ 
	return m_pPanel->getCurveName(id); 
}

void Object::toggleCurve(int id)
{
	m_pPanel->toggleCurve(id); 
}

void Object::setControlValue(int id, float v)
{
	m_pPanel->setControlValue(id, v);
}

void Object::setCurveValue(int id, float v)
{
	m_pPanel->setCurveValue(id, v);
}

void Object::refresh()
{ 
	m_pScene->refresh(); 
};

bool Object::null_init(void *v)
{
	return true;
}

void Object::null_redraw(int id,int drawMode)
{
}

