/* -*- compile-command: "msdev modeler.dsp /make \"Modeler - Win32 Debug\"" -*-
 */

#ifndef _MODELER_H
#define _MODELER_H

#include <stdio.h>
#include <assert.h>
#include <string.h>

#include <FL/gl.h>
#include <GL/glu.h>

#include "modeler.h"

#ifdef WIN32
#define DLLEXPORT __declspec( dllexport )
#else
#define DLLEXPORT
#endif

// grrr.  why does Microsoft not put the value of pi in any of its
//   standard include files? 
#define M_PI 3.14159265358979323846

extern int g_drawmode;

extern GLfloat mat_ambient[4];
extern GLfloat mat_diffuse[4];
extern GLfloat mat_specular[4];
extern GLfloat mat_shininess;

		   
// g_quality controls polygonalization of the sphere and cylinder
//   primitives.
//   higher g_quality --> more polygons --> prettier but slower. 
extern int g_quality;

//*
//* function prototypes:  anything marked with DLLEXPORT
//* is something you can call from your modeler function(s).  anything
//* without this declaration is only needed internally, and you can't
//* call it (so don't try!).


/** general setup and animation stuff **/

/** set drawmode and drawquality, and ray file name **/
extern "C" {
void DLLEXPORT setdrawmode( int drawmode, char *rayfilename );
}

/** material properties **/
void DLLEXPORT ambient_color( double r, double g, double b );
void DLLEXPORT diffuse_color( double r, double g, double b );
void DLLEXPORT specular_color( double r, double g, double b );
void DLLEXPORT shininess( double s );


/** geometric primitives **/

// sphere of radius r 
void DLLEXPORT sphere( double r );

// axis-aligned box from origin to (x,y,z).
void DLLEXPORT box( double x, double y, double z );

// axis-aligned texture box from origin to (x,y,z).
void DLLEXPORT textureBox( double x, double y, double z );

// cylinder from z=0 to z=h with radius r1 at origin and r2 at z=h. 
void DLLEXPORT cylinder( double h, double r1, double r2 );

// triangle with three given vertices. 
void DLLEXPORT triangle( double x1, double y1, double z1,
				       double x2, double y2, double z2,
				       double x3, double y3, double z3 );

// bitmap handlers

DLLEXPORT unsigned char *readBMP(char *fname, int& width, int& height);
DLLEXPORT void writeBMP(char *iname, int width, int height, unsigned char *data); 


extern int import_objID; 

/* put the MODEL macro (with the name of your model) in your modeler
   file to create a boilerplate initialization function. */

/* Note: Don't use the characters # and @ in the names of your controls */

#define MODEL(name) \
extern "C" { \
bool DLLEXPORT init( void * ); \
void DLLEXPORT redraw_driver( int, int ); \
void DLLEXPORT patchControl( lpGroupFunc,lpCurveFunc, lpScaleFunc, lpCheckboxFunc, \
							 lpMenuFunc, lpGetfFunc, lpGetiFunc, lpGetbFunc, \
							 lpGetTimeFunc, lpPtrFunc, lpGetPtrFunc lpGetPtr ); \
void DLLEXPORT getExportModelName( char * ); \
void DLLEXPORT assignID( int ); \
}; \
lpGroupFunc group; \
lpScaleFunc scale; \
lpCurveFunc curve; \
lpCheckboxFunc checkbox; \
lpMenuFunc menu; \
lpPtrFunc ptr; \
lpGetTimeFunc get_time; \
lpGetfFunc get_control_f_id; \
lpGetiFunc get_control_i_id; \
lpGetbFunc get_control_b_id; \
lpGetPtrFunc get_ptr_id; \
int import_objID; \
void redraw(int); \
void redraw_driver(int obj_id, int drawmode) \
{ \
	import_objID=obj_id; \
	redraw(drawmode); \
} \
void patchControl(lpGroupFunc pGroup, lpCurveFunc pCurve, lpScaleFunc pScale, lpCheckboxFunc pCheckbox, \
				  lpMenuFunc pMenu, lpGetfFunc pGetf, lpGetiFunc pGeti, lpGetbFunc pGetb, \
				  lpGetTimeFunc pGetTime, lpPtrFunc pPtr, lpGetPtrFunc pGetPtr) \
{ \
	group = pGroup; \
	curve = pCurve; \
	scale = pScale; \
	checkbox = pCheckbox; \
	menu = pMenu; \
	ptr = pPtr; \
	get_time = pGetTime; \
	get_control_f_id = pGetf; \
	get_control_i_id = pGeti; \
	get_control_b_id = pGetb; \
	get_ptr_id = pGetPtr; \
} \
void getExportModelName( char *str ) \
{ \
	strcpy(str, name); \
} \
void assignID(int id) \
{ \
	import_objID = id; \
} \
double get_control_d(int ctrl_id) \
{ \
	return (double)get_control_f_id(import_objID, ctrl_id); \
} \
float get_control_f(int ctrl_id) \
{ \
	return get_control_f_id(import_objID, ctrl_id); \
} \
int get_control_i(int ctrl_id) \
{ \
	return get_control_i_id(import_objID, ctrl_id); \
} \
bool get_control_b(int ctrl_id) \
{ \
	return get_control_b_id(import_objID, ctrl_id); \
} \
void *get_ptr(int ptr_id) \
{ \
	return get_ptr_id(import_objID, ptr_id); \
}

#endif 


    
